from ftw.tabbedview.browser.tabbed import TabbedView
from ftw.table import helper
from ftw.workspace.browser.tab import Tab
from ftw.workspace import _


class WorkspacesView(TabbedView):
    """Tabbed workspaces overview"""

    def get_tabs(self):
        return [{'id': 'workspaces', 'class': ''},
                {'id': 'documents', 'class': ''},
                {'id': 'events', 'class': ''},
                {'id': 'activity', 'class': 'searchform-hidden'},
                ]


class WorkspacesTab(Tab):

    types = 'Workspace'
    sort_on = 'sortable_title'
    show_selects = False
    show_menu = False

    columns = (
        {'column': 'Title',
         'column_title': _(u'column_title', default=u'Title'),
         'sort_index': 'sortable_title',
         'transform': helper.linked},

        {'column': 'modified',
         'column_title': _(u'column_modified', default=u'modified'),
         'transform': helper.readable_date_text,
         'width': 80},

        {'column': 'Creator',
         'column_title': _(u'column_creator', default=u'Creator'),
         'sort_index': 'sortable_creator',
         'transform': helper.readable_author}, )



from ftw.upgrade import UpgradeStep
from Products.CMFCore.permissions import View


class InstallConfigureParticipation(UpgradeStep):
    """Install configure participation.
    """

    def __call__(self):
        version = self.portal_setup.getLastVersionForProfile(
            'ftw.participation:default')

        if version is None or version == 'unknown':
            self.setup_install_profile('profile-ftw.participation:default')

        self.install_upgrade_profile(steps=['plone.app.registry'])

        self.add_pariticipants_tab()

    def add_pariticipants_tab(self):
        self.actions_add_type_action(
            portal_type='Workspace',
            after=None,
            action_id='participants',
            **{'category': 'tabbedview-tabs',
               'action': 'string:${object_url}?view=participants',
               'visible': True,
               'permissions': (View, )})



from Products.CMFCore.utils import getToolByName
from ftw.builder import Builder
from ftw.builder import create
from ftw.workspace.testing import FTW_WORKSPACE_INTEGRATION_TESTING
from plone.app.testing import login
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from unittest2 import TestCase


class TestWorkspace(TestCase):

    layer = FTW_WORKSPACE_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        setRoles(self.portal, TEST_USER_ID, ['Manager'])

    def test_searchable_text_contains_workspace_creator(self):
        """
        The full name of the creator of a workspace has to be in the
        searchable text index so the workspace overview listing can
        be filtered by creator.
        """
        folder = create(Builder('folder'))
        hugo = create(
            Builder('user')
            .named('Hugo', 'Boss')
            .with_roles('Contributor', on=folder)
        )

        login(self.portal, hugo.getId())
        workspace = create(
            Builder('workspace').within(folder).titled('My Workspace')
        )

        portal_catalog = getToolByName(self.layer['portal'], 'portal_catalog')
        rid = portal_catalog.getrid('/'.join(workspace.getPhysicalPath()))
        index_data = portal_catalog.getIndexDataForRID(rid)

        expected = {'boss', 'hugo', 'my', 'workspace'}
        actual = set(index_data['SearchableText'])
        self.assertTrue(expected.issubset(actual))



from Acquisition import aq_parent, aq_base, aq_inner
from Products.CMFCore.utils import getToolByName
from Products.CMFPlone.interfaces.factory import IFactoryTool
from ftw.workspace.utils import find_workspace
from zope.component.hooks import getSite
from zope.interface import implements
from zope.schema.interfaces import IVocabularyFactory
from zope.schema.vocabulary import SimpleVocabulary


class PrincipalVocabulary(SimpleVocabulary):
    """Vocabulary of a list of users. The userids are
    passed to __init__.
    """

    def __init__(self, userids):
        self.userids = userids
        super(PrincipalVocabulary, self).__init__(tuple(self._get_terms()))

    def _get_terms(self):
        portal = getSite()
        acl_users = getToolByName(portal, 'acl_users')

        users_dict = {}
        for id_ in self.userids:
            user = acl_users.getUserById(id_)

            catalog = getToolByName(portal, 'portal_catalog')
            brains = catalog(dict(UID=id_))
            if user is not None:
                users_dict[id_] = user.getProperty('fullname', id_)

            elif len(brains):
                brain = brains[0]
                users_dict[brain.UID] = "%s (Kontakt)" % brain.Title

        for id_, fullname in sorted(
            users_dict.items(), key=lambda item: item[1]):

            yield self.createTerm(id_, str(id_), fullname)


class AssignableUsersVocabulary(object):
    """Vocabulary of all users which participates in this workspace.
    """

    implements(IVocabularyFactory)

    def __call__(self, context, membersonly=False):
        workspace = find_workspace(context)
        catalog = getToolByName(context, 'portal_catalog')
        gtool = getToolByName(context, 'portal_groups')
        # users = getUtility(
        #     IVocabularyFactory,
        #     name='plone.principalsource.Users',
        #     context=context)(context)

        if not workspace:
            workspace = context

        # Create a list of user depending on local roles and inherited
        # roles. Also read out users from groups
        users = set([])
        groups = set([])

        # Walk upwards until reach portal root or role acquire check fails
        portal = workspace.portal_url.getPortalObject()
        cont = True
        while cont:
            if context == portal:
                break

            if IFactoryTool.providedBy(context):
                # this is portal_factory - skip it
                context = aq_parent(aq_inner(context))
                continue

            if IFactoryTool.providedBy(aq_parent(context)):
                # this is the 'Ticket Box' temp folder
                # within portal_factory - skip it
                context = aq_parent(aq_inner(context))
                continue

            userroles = portal.acl_users._getLocalRolesForDisplay(context)
            # Use dict's to auto. prevent duplicated entries
            for user, roles, role_type, name in userroles:
                if role_type == u'user' and name not in users:
                    users.add(name)
                elif role_type == u'group' and user not in groups:
                    groups.add(name)

            if getattr(aq_base(context), '__ac_local_roles_block__', None):
                cont = False
            else:
                context = aq_parent(context)

        # Go throught groups an add their containing users to the user list
        for groupid in groups:
            group = gtool.getGroupById(groupid)
            if group:
                members = set(group.getGroupMemberIds())
            else:
                continue
            # Put together
            users = users.union(members)

        result = users

        if not membersonly:
            query = dict(
                portal_type='Contact',
                path='/'.join(workspace.getPhysicalPath()),
                sort_on='sortable_title')

            for brain in catalog(query):
                result.add(brain.UID)

        return PrincipalVocabulary(result)


AssignableUsersVocabularyFactory = AssignableUsersVocabulary()



from ftw.workspace.browser.previews.default import DefaultPreview
from ftw.workspace.interfaces import IWorkspacePreview
from Products.CMFCore.utils import getToolByName
from zope.component import adapts
from zope.interface import implements
from zope.interface import Interface
from Products.ATContentTypes.interfaces.file import IATFile


class ZipPreview(DefaultPreview):

    implements(IWorkspacePreview)
    adapts(IATFile, Interface)

    def full_url(self):
        portal_url = getToolByName(self.context, 'portal_url')
        return '{0}/++resource++ftw.workspace-resources/zip.png'.format(
            portal_url())


class TxtPreview(DefaultPreview):

    implements(IWorkspacePreview)
    adapts(IATFile, Interface)

    def full_url(self):
        portal_url = getToolByName(self.context, 'portal_url')
        return '{0}/++resource++ftw.workspace-resources/txt.png'.format(
            portal_url())



from DateTime import DateTime
from ftw.pdfgenerator.interfaces import IBuilder
from ftw.pdfgenerator.interfaces import ILaTeXView
from ftw.testing import MockTestCase
from ftw.workspace.interfaces import IWorkspace
from ftw.workspace.interfaces import IWorkspaceDetailsListingProvider
from ftw.workspace.latex.layout import WorkspaceLayout
from ftw.workspace.latex.views import FilesListing
from ftw.workspace.latex.views import WorkspaceDetailsView
from ftw.workspace.testing import LATEX_ZCML_LAYER
from mocker import ANY
from zope.component import getMultiAdapter
from zope.interface.verify import verifyClass


class ListingTestBase(MockTestCase):

    def setUp(self):
        MockTestCase.setUp(self)

        self.context = self.providing_stub(IWorkspace)
        self.expect(self.context.getPhysicalPath()).result(
            ['', 'path', 'to', 'workspace'])
        self.expect(self.context.getOwner(0).getId()).result('john.doe')

        self.response = self.stub()
        self.expect(self.response.getHeader(ANY))
        self.expect(self.response.setHeader(ANY, ANY))
        self.request = self.create_dummy(debug=True,
                                         response=self.response)

        self.builder = self.stub_interface(IBuilder)
        self.layout = WorkspaceLayout(
            self.context, self.request, self.builder)
        self.view = WorkspaceDetailsView(
            self.context, self.request, self.layout)

        portal_catalog = self.stub()
        self.mock_tool(portal_catalog, 'portal_catalog')
        self.brains = []
        self.expect(portal_catalog({'path': '/path/to/workspace',
                                    'portal_type': ['File'],
                                    'sort_on': 'created',
                                    'sort_order': 'reverse'})).call(
            lambda q: self.brains)

        acl_users = self.stub()
        self.mock_tool(acl_users, 'acl_users')
        self.expect(acl_users.getUserById('john.doe').getProperty(
                'fullname', 'john.doe')).result('John Doe')



class TestWorkspaceDetailsView(ListingTestBase):

    layer = LATEX_ZCML_LAYER

    def test_component_is_registered(self):
        self.replay()
        view = getMultiAdapter((self.context, self.request, self.layout),
                               ILaTeXView)
        self.assertEqual(type(view), WorkspaceDetailsView)

    def test_implements_interface(self):
        self.replay()
        self.assertTrue(ILaTeXView.implementedBy(WorkspaceDetailsView))
        verifyClass(ILaTeXView, WorkspaceDetailsView)

    def test_metadata_in_view(self):
        self.expect(self.context.Title()).result('johns workspace')
        self.expect(self.context.Description()).result('this is my workspace')
        self.expect(self.context.getText()).result('the long description')

        self.replay()

        view = getMultiAdapter((self.context, self.request, self.layout),
                               ILaTeXView)
        latex = view.render()

        self.assertIn('johns workspace', latex)
        self.assertIn('this is my workspace', latex)
        self.assertIn('the long description', latex)
        self.assertIn('John Doe', latex)

    def test_full_rendering(self):
        self.brains = [
            self.create_dummy(Title='my document',
                              effective=DateTime('05/23/2010'),
                              modified=DateTime('06/10/2010'),
                              Creator='john.doe')]

        self.expect(self.context.Title()).result('johns workspace')
        self.expect(self.context.Description()).result('this is my workspace')
        self.expect(self.context.getText()).result('the long description')

        portal_setup = self.stub()
        self.expect(portal_setup.getLastVersionForProfile(
                'ftw.file:default')).result('unknown')
        self.mock_tool(portal_setup, 'portal_setup')

        self.replay()

        view = getMultiAdapter((self.context, self.request, self.layout),
                               ILaTeXView)
        latex = view.render()

        self.assertIn('johns workspace', latex)
        self.assertIn('my document', latex)


class TestFilesListing(ListingTestBase):

    layer = LATEX_ZCML_LAYER

    def test_component_is_registered(self):
        self.replay()
        listing = getMultiAdapter(
            (self.context, self.request, self.layout, self.view),
            IWorkspaceDetailsListingProvider,
            name='files-listing')

        self.assertEqual(type(listing), FilesListing)

    def test_implements_interface(self):
        self.replay()
        self.assertTrue(IWorkspaceDetailsListingProvider.implementedBy(
                FilesListing))

        verifyClass(IWorkspaceDetailsListingProvider, FilesListing)

    def test_get_sort_key(self):
        self.replay()
        listing = getMultiAdapter(
            (self.context, self.request, self.layout, self.view),
            IWorkspaceDetailsListingProvider,
            name='files-listing')
        self.assertEqual(listing.get_sort_key(), 10)

    def test_get_title(self):
        self.replay()
        listing = getMultiAdapter(
            (self.context, self.request, self.layout, self.view),
            IWorkspaceDetailsListingProvider,
            name='files-listing')
        self.assertEqual(listing.get_title(), 'Files')

    def test_get_items__WITHOUT_FTW_FILE(self):
        self.brains = [
            self.create_dummy(Title='foo',
                              effective=DateTime('05/23/2010'),
                              modified=DateTime('06/10/2010'),
                              Creator='john.doe')]

        portal_setup = self.stub()
        self.expect(portal_setup.getLastVersionForProfile(
                'ftw.file:default')).result('unknown')
        self.mock_tool(portal_setup, 'portal_setup')

        self.replay()
        listing = getMultiAdapter(
            (self.context, self.request, self.layout, self.view),
            IWorkspaceDetailsListingProvider,
            name='files-listing')

        self.assertEqual(list(listing.get_items()), [
                {'title': 'foo',
                 'effective': '23.05.2010',
                 'modified': '10.06.2010',
                 'creator': 'John Doe'}])

    def test_get_items__FTW_FILE(self):
        self.brains = [
            self.create_dummy(Title='foo',
                              effective=DateTime('05/23/2010'),
                              modified=DateTime('06/10/2010'),
                              documentDate=DateTime('06/13/2010'),
                              Creator='john.doe')]

        portal_setup = self.stub()
        self.expect(portal_setup.getLastVersionForProfile(
                'ftw.file:default')).result('1.0')
        self.mock_tool(portal_setup, 'portal_setup')

        self.replay()
        listing = getMultiAdapter(
            (self.context, self.request, self.layout, self.view),
            IWorkspaceDetailsListingProvider,
            name='files-listing')

        self.assertEqual(list(listing.get_items()), [
                {'title': 'foo',
                 # When ftw.file is there, we use the documentDate as
                 # effective.
                 'effective': '13.06.2010',
                 'modified': '10.06.2010',
                 'creator': 'John Doe'}])

    def test_rendering(self):
        self.brains = [
            self.create_dummy(Title='foo',
                              effective=DateTime('05/23/2010'),
                              modified=DateTime('06/10/2010'),
                              Creator='john.doe')]

        portal_setup = self.stub()
        self.expect(portal_setup.getLastVersionForProfile(
                'ftw.file:default')).result('unknown')
        self.mock_tool(portal_setup, 'portal_setup')

        self.replay()
        listing = getMultiAdapter(
            (self.context, self.request, self.layout, self.view),
            IWorkspaceDetailsListingProvider,
            name='files-listing')

        latex = listing.get_listing()
        self.assertIn(r'\begin{tabular}', latex)
        self.assertIn(r'foo', latex)
        self.assertIn(r'23.05.2010', latex)
        self.assertIn(r'10.06.2010', latex)
        self.assertIn(r'John Doe', latex)



"""Common configuration constants
"""

PROJECTNAME = 'ftw.workspace'

ADD_PERMISSIONS = {
    'Workspace': 'ftw.workspace: Add Workspace',
    'TabbedViewFolder': 'ATContentTypes: Add Folder',
}

INDEXES = (("sortable_creator", "FieldIndex"),
           ("getContentType", "FieldIndex"),
           ("ownerid", "FieldIndex"), )

METADATA = ("getMeeting_type", )
DEFAULT_TINYMCE_ALLOWED_BUTTONS = (
    'bg-basicmarkup',
    'bold-button',
    'italic-button',
    'list-ol-addbutton',
    'list-ul-addbutton',
    'definitionlist',
    'linklibdrawer-button',
    'removelink-button', )



from Acquisition import aq_inner, aq_parent
from ftw.workspace import _
from OFS.CopySupport import CopyError, ResourceLockedError
from plone.formwidget.contenttree import ContentTreeFieldWidget
from plone.formwidget.contenttree import ObjPathSourceBinder
from plone.z3cform import layout
from Products.CMFCore.utils import getToolByName
from Products.statusmessages.interfaces import IStatusMessage
from z3c.form import form, field
from z3c.form import validator
from z3c.form.interfaces import HIDDEN_MODE
from z3c.relationfield.schema import RelationChoice
from zope import schema
from zope.component import provideAdapter
from zope.interface import Interface, Invalid
from zope.publisher.browser import BrowserView
import z3c.form


class IMoveItemsSchema(Interface):
    destination_folder = RelationChoice(
        title=_('label_destination', default="Destination"),
        description=_('help_destination',
                      default="Select the destination container."),
        source=ObjPathSourceBinder(
            is_folderish=True,
            navigation_tree_query=None),
        required=True,
        )
    #We Use TextLine here because Tuple and List have no hidden_mode.
    request_paths = schema.TextLine(title=u"request_paths")


class MoveItemsForm(form.Form):

    fields = field.Fields(IMoveItemsSchema)
    fields['destination_folder'].widgetFactory = ContentTreeFieldWidget
    ignoreContext = True
    label = _('heading_move_items', default="Move Items")

    def updateWidgets(self):
        super(MoveItemsForm, self).updateWidgets()
        self.widgets['request_paths'].mode = HIDDEN_MODE
        value = self.request.get('paths')
        if value:
            self.widgets['request_paths'].value = ';;'.join(value)

    @z3c.form.button.buttonAndHandler(_(u'button_move',
                                        default=u'Move'))
    def handle_submit(self, action):
        data, errors = self.extractData()
        if len(errors):
            return

        source = data['request_paths'].split(';;')
        destination = data['destination_folder']
        failed_objects = []
        failed_resource_locked_objects = []
        moved_items = 0

        for path in source:

            portal = getToolByName(
                self.context, 'portal_url').getPortalObject()

            # Get source object
            src_object = portal.unrestrictedTraverse(
                path.encode('utf-8'))

            # Get parent object
            source_container = aq_parent(aq_inner(
                portal.unrestrictedTraverse(path.encode('utf-8'))))

            src_name = src_object.title
            src_id = src_object.id

            try:
                # Try to cut and paste object
                clipboard = source_container.manage_cutObjects(src_id)
                destination.manage_pasteObjects(clipboard)
                moved_items += 1

            except ResourceLockedError:
                # The object is locket over webdav
                failed_resource_locked_objects.append(src_name)
                continue

            except (ValueError, CopyError):
                # Catch exception and add title to a list of failed objects
                failed_objects.append(src_name)
                continue

        self.create_statusmessages(
            moved_items,
            failed_objects,
            failed_resource_locked_objects, )

        self.request.RESPONSE.redirect(destination.absolute_url())

    @z3c.form.button.buttonAndHandler(_(u'button_cancel',
                                        default=u'Cancel'))
    def handle_cancel(self, action):
        return self.request.RESPONSE.redirect(self.context.absolute_url())

    def create_statusmessages(
        self,
        moved_items,
        failed_objects,
        failed_resource_locked_objects
    ):
        """ Create statusmessages with errors and infos af the move-process
        """
        if moved_items:
            msg = _(u'${moved_items} Elements were moved successfully',
                    mapping=dict(moved_items=moved_items))
            IStatusMessage(self.request).addStatusMessage(
                msg, type='info')

        if failed_objects:
            msg = _(u'Failed to move following objects: ${failed_objects}',
                    mapping=dict(failed_objects=',<br>'.join(failed_objects)))
            IStatusMessage(self.request).addStatusMessage(
                msg, type='error')

        if failed_resource_locked_objects:
            msg = _(
                u'''Failed to move following objects: ''' +
                '''${failed_objects}. Locked via WebDAV''',
                mapping=dict(failed_objects=',<br>'.join(
                    failed_resource_locked_objects)))
            IStatusMessage(self.request).addStatusMessage(
                msg, type='error')


class MoveItemsFormView(layout.FormWrapper, BrowserView):
    """ View to move selected items into another location
    """

    form = MoveItemsForm

    def __init__(self, context, request):
        layout.FormWrapper.__init__(self, context, request)

    def render(self):
        if not self.request.get('paths') and not \
                self.form_instance.widgets['request_paths'].value:
            msg = _(u'You have not selected any items')
            IStatusMessage(self.request).addStatusMessage(
                msg, type='error')

            # redirect to the right tabbedview_tab
            if self.request.form.get('orig_template'):

                return self.request.RESPONSE.redirect(
                    self.request.form.get('orig_template'))
            # fallback documents tab
            else:
                return self.request.RESPONSE.redirect(
                    '%s#documents' % self.context.absolute_url())
        return super(MoveItemsFormView, self).render()


class NotInContentTypes(Invalid):
    __doc__ = _(u"It isn't allowed to add such items there")


class DestinationValidator(validator.SimpleFieldValidator):
    """Validator for destination-path.
    We check the destinations allowed content-type. If one or more source-types
    are not allowed in the destination, we raise an error
    """

    def validate(self, value):
        super(DestinationValidator, self).validate(value)

        # Allowed contenttypes for destination-folder
        allowed_types = [t.getId() for t in value.allowedContentTypes()]

        # Paths to source object
        source = self.view.widgets['request_paths'].value.split(';;')

        # Get source-brains
        portal_catalog = getToolByName(self.context, 'portal_catalog')
        src_brains = portal_catalog(path={'query': source, 'depth': 0})
        failed_objects = []

        # Look for invalid contenttype
        for src_brain in src_brains:
            if not src_brain.portal_type in allowed_types:
                failed_objects.append(src_brain.Title.decode('utf8'))

        # If we found one or more invalid contenttypes, we raise an error
        if failed_objects:
            raise NotInContentTypes(
                _(u"error_NotInContentTypes ${failed_objects}",
                  default=u"It isn't allowed to add such items there: " +
                  "${failed_objects}", mapping=dict(
                      failed_objects=',<br>'.join(failed_objects))))

validator.WidgetValidatorDiscriminators(
    DestinationValidator, field=IMoveItemsSchema['destination_folder'])
provideAdapter(DestinationValidator)



from ftw.builder.session import BuilderSession
from ftw.builder.testing import BUILDER_LAYER
from ftw.builder.testing import set_builder_session_factory
from ftw.tabbedview.interfaces import ITabbedView
from plone.app.testing import applyProfile
from plone.app.testing import FunctionalTesting
from plone.app.testing import IntegrationTesting
from plone.app.testing import PLONE_FIXTURE
from plone.app.testing import PloneSandboxLayer
from plone.registry.interfaces import IRegistry
from plone.testing import Layer
from plone.testing import z2
from plone.testing import zca
from zope.component import getUtility
from zope.configuration import xmlconfig
import egov.contactdirectory.tests.builders
import ftw.meeting.tests.builders
import ftw.workspace.tests.builders


def functional_session_factory():
    sess = BuilderSession()
    sess.auto_commit = True
    return sess


class FtwWorkspaceLayer(PloneSandboxLayer):

    defaultBases = (PLONE_FIXTURE, BUILDER_LAYER)

    def setUpZope(self, app, configurationContext):
        xmlconfig.string(
            '<configure xmlns="http://namespaces.zope.org/zope">'
            '  <include package="z3c.autoinclude" file="meta.zcml" />'
            '  <includePlugins package="plone" />'
            '  <includePluginsOverrides package="plone" />'
            '  <include package="Products.CMFPlacefulWorkflow" />'
            '</configure>',
            context=configurationContext)

        # DatagridField doesn't define the auto-include entry point
        import Products.DataGridField
        xmlconfig.file('configure.zcml',
                       Products.DataGridField,
                       context=configurationContext)

        z2.installProduct(app, 'ftw.workspace')
        z2.installProduct(app, 'ftw.file')
        z2.installProduct(app, 'ftw.meeting')
        z2.installProduct(app, 'Products.DataGridField')
        z2.installProduct(app, 'egov.contactdirectory')
        z2.installProduct(app, 'Products.CMFPlacefulWorkflow')
        # Dep. of egov.contactdirectory
        z2.installProduct(app, 'ftw.contentpage')
        z2.installProduct(app, 'simplelayout.types.common')

    def setUpPloneSite(self, portal):
        # Install into Plone site using portal_setup
        applyProfile(portal, 'ftw.workspace:default')
        applyProfile(portal, 'ftw.workspace:contact')

        # Disable extjs integration for tests.
        registry = getUtility(IRegistry)
        reg_proxy = registry.forInterface(ITabbedView)
        reg_proxy.extjs_enabled = False


FTW_WORKSPACE_FIXTURE = FtwWorkspaceLayer()
FTW_WORKSPACE_INTEGRATION_TESTING = IntegrationTesting(
    bases=(FTW_WORKSPACE_FIXTURE, ), name="FtwWorkspace:Integration")

FTW_WORKSPACE_FUNCTIONAL_TESTING = FunctionalTesting(
    bases=(FTW_WORKSPACE_FIXTURE,
           set_builder_session_factory(functional_session_factory)),
    name="FtwWorkspace:Functional")


class LatexZCMLLayer(Layer):
    """A layer which only sets up the zcml, but does not start a zope
    instance.
    """

    defaultBases = (zca.ZCML_DIRECTIVES, )

    def testSetUp(self):
        self['configurationContext'] = zca.stackConfigurationContext(
            self.get('configurationContext'))

        import zope.traversing
        xmlconfig.file('configure.zcml', zope.traversing,
                       context=self['configurationContext'])

        import ftw.pdfgenerator.tests
        xmlconfig.file('test.zcml', ftw.pdfgenerator.tests,
                       context=self['configurationContext'])

        import ftw.pdfgenerator
        xmlconfig.file('configure.zcml', ftw.pdfgenerator,
                       context=self['configurationContext'])

        import ftw.workspace.latex
        xmlconfig.file('configure.zcml', ftw.workspace.latex,
                       context=self['configurationContext'])

    def testTearDown(self):
        del self['configurationContext']


LATEX_ZCML_LAYER = LatexZCMLLayer()



from ftw.upgrade import UpgradeStep


class ConfigureInstallExtJsAndQuickupload(UpgradeStep):
    """Configure install extjs and quickupload.
    """

    def __call__(self):
        extjs_version = self.portal_setup.getLastVersionForProfile(
            'ftw.tabbedview:extjs')
        if extjs_version is None or extjs_version == 'unknown':
            self.setup_install_profile('profile-ftw.tabbedview:extjs')

        quickuplad_version = self.portal_setup.getLastVersionForProfile(
            'ftw.tabbedview:quickupload')
        if quickuplad_version is None or quickuplad_version == 'unknown':
            self.setup_install_profile('profile-ftw.tabbedview:quickupload')



from ftw.workspace.browser.previews.default import DefaultPreview
from ftw.workspace.interfaces import IWorkspacePreview
from plone.app.blob.interfaces import IATBlobImage
from zope.component import adapts
from zope.interface import implements
from zope.interface import Interface


class ImagePreview(DefaultPreview):
    """Image preview"""

    implements(IWorkspacePreview)
    adapts(IATBlobImage, Interface)

    @property
    def _primary_field(self):
        return self.context.getPrimaryField().getName()

    def preview(self):
        scale = self.context.restrictedTraverse('@@images')
        width, height = self.get_scale_properties()

        return scale.scale(
            self._primary_field,
            width=width,
            height=height,
            direction='down').tag()

    def full_url(self):
        return '{0}/images/{1}'.format(self.context.absolute_url(),
                                self._primary_field)



from ftw.tabbedview.browser import listing
from ftw.table import helper
from ftw.workspace import _
from Products.CMFCore.utils import getToolByName
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from zope.i18nmessageid import MessageFactory
from Products.Five.browser import BrowserView


plone_locales_mf = MessageFactory('plonelocales')


class EventsTab(listing.CatalogListingView):
    types = ['Meeting', 'Poodle', 'Event']

    sort_on = 'start'
    sort_order = 'reverse'

    show_selects = False
    show_menu = False

    columns = ({'column': 'start',
                'column_index': 'start',
                'column_title': _(u'label_eventstab_start'),
                'transform': helper.readable_date_text,
                'width': 80},

               {'column': 'Title',
                'column_index': 'sortable_title',
                'column_title': _(u'label_eventstab_title'),
                'sort_index': 'sortable_title',
                'transform': helper.linked},


               {'column': 'getMeeting_type',
                'column_title': _(u'label_eventstab_type',
                                  default=u'Type'),
                'transform': helper.translated_string('ftw.meeting'),
                'width': 80},


               {'column': 'Creator',
                'column_index': 'sortable_creator',
                'column_title': _(u'label_eventstab_creator'),
                'transform': helper.readable_author}, )

    template = ViewPageTemplateFile('events.pt')


class EventsCalendarTab(BrowserView):

    template = ViewPageTemplateFile('eventscalendar.pt')
    types = []

    def __init__(self, *args, **kwargs):
        super(EventsCalendarTab, self).__init__(*args, **kwargs)
        self.js_config = None

    def __call__(self):
        portal = getToolByName(self.context, 'portal_url').getPortalObject()
        self.js_config = portal.unrestrictedTraverse('ftwcalendar_config.js')()
        return self.template()



from ftw.upgrade import UpgradeStep


class InstallAndConfigureFtwMeeting(UpgradeStep):
    """Install and configure ftw meeting.
    """

    def __call__(self):
        version = self.portal_setup.getLastVersionForProfile(
            'ftw.meeting:default')

        if version is None or version == 'unknown':
            self.setup_install_profile('profile-ftw.meeting:default')

        self.install_upgrade_profile(steps=['typeinfo'])



from Acquisition import aq_inner, aq_parent
from Products.CMFPlone.interfaces.siteroot import IPloneSiteRoot
from ftw.workspace.interfaces import IWorkspace
from ftw.workspace.config import DEFAULT_TINYMCE_ALLOWED_BUTTONS
from plone.registry.interfaces import IRegistry
from zope.component import getUtility
from Products.CMFCore.utils import getToolByName


def has_ftwfile(context):
    portal_setup = getToolByName(context, 'portal_setup')
    profile_version = portal_setup.getLastVersionForProfile('ftw.file:default')
    return profile_version != 'unknown'


def find_workspace(context):
    """Walks up and returns the next parent workspace (IWorkspace) or
    returns None if plone site is reached.
    """

    obj = context
    while True:
        if IWorkspace.providedBy(obj):
            return obj

        elif IPloneSiteRoot.providedBy(obj):
            return None

        parent = aq_parent(aq_inner(obj))
        if parent == obj:
            raise ValueError('Somethings wrong: cannot walk up further')
        else:
            obj = parent


class TinyMCEAllowedButtonsConfigurator(object):
    """
    The TinyMCE RichWidget expects the button configuration attributes
    to be a iterable object.

    The configuration is stored in the registry, so we need to do a
    little workaround.
    """

    def __init__(self):
        self._data = None

    def __iter__(self):
        """This is a iterator, so it returns itself.
        """
        return self

    def next(self):
        """Iterate over the result of `load_data`. When the end is reached,
        raise StopIteration as defined for iterable objects. But when
        starting iteration again, retrieve the configuration and start again
        at the beginning.
        """

        if self._data is None:
            self._data = list(self.load_data())

        if len(self._data) > 0:
            return self._data.pop(0)

        else:
            self._data = None
            raise StopIteration()

    def load_data(self):
        registry = getUtility(IRegistry)
        if 'ftw.workspace.allow_buttons' in registry:
            allow_buttons = registry['ftw.workspace.allow_buttons']
            if allow_buttons:
                return allow_buttons

        # Fallback
        return DEFAULT_TINYMCE_ALLOWED_BUTTONS


def get_creator_fullname(obj):
    creator = obj.Creator()
    pas_tool = getToolByName(obj, 'acl_users')
    user = pas_tool.getUserById(creator)
    if not user:
        return creator
    fullname = user.getProperty('fullname', creator)
    if fullname:
        return fullname
    return creator



from ftw.pdfgenerator.interfaces import IBuilder
from ftw.pdfgenerator.interfaces import ILaTeXLayout
from ftw.pdfgenerator.tests import test_customizable_layout
from ftw.workspace.interfaces import IWorkspace
from ftw.workspace.latex.layout import WorkspaceLayout
from ftw.workspace.testing import LATEX_ZCML_LAYER
from zope.component import getMultiAdapter


class TestWorkspaceLayout(test_customizable_layout.TestCustomizableLayout):

    layout_class = WorkspaceLayout
    layer = LATEX_ZCML_LAYER

    def setUp(self):
        super(test_customizable_layout.TestCustomizableLayout, self).setUp()
        self.context = self.providing_stub(IWorkspace)
        self.builder = self.stub_interface(IBuilder)
        super(TestWorkspaceLayout, self).setUp(context=self.context,
                                               builder=self.builder)

        self.portal_languages = self.stub()
        self.mock_tool(self.portal_languages, 'portal_languages')
        self.expect(self.portal_languages.getPreferredLanguage()).result(
            'en')

    def test_get_render_arguments(self):
        self.replay()

        layout = getMultiAdapter((self.context, self.request, self.builder),
                            ILaTeXLayout)
        args = layout.get_render_arguments()

        self.assertIn('pageLabel', args)
        self.assertIn('printedLabel', args)
        self.assertIn('printedLabel', args)

    def test_layout_renders(self):
        self.replay()
        layout = getMultiAdapter((self.context, self.request, self.builder),
                                 ILaTeXLayout)
        latex = layout.render_latex('CONTENT LATEX')

        self.assertIn('\documentclass[a4paper,10pt]{article}', latex)
        self.assertIn('[english]{babel}', latex)
        self.assertIn('{fancyhdr}', latex)
        self.assertIn('{lastpage}', latex)
        self.assertIn('{hyperref}', latex)
        self.assertIn('CONTENT LATEX', latex)



from ftw.workspace.interfaces import IWorkspace
from ftw.workspace.utils import get_creator_fullname
from plone.indexer.decorator import indexer
from Products.ATContentTypes.interfaces.interfaces import IATContentType


@indexer(IWorkspace)
def ownerid(object_, **kw):
    """indexes the userid of the object owner"""
    userid = object_.getOwner(0).getId()
    return userid and userid or ''


@indexer(IATContentType)
def sortable_creator(object_, **kw):
    return get_creator_fullname(object_)



from ftw.builder import Builder
from ftw.builder import create
from ftw.workspace.testing import FTW_WORKSPACE_INTEGRATION_TESTING
from operator import methodcaller
from plone.app.testing import login
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone.app.testing import TEST_USER_NAME
from plone.uuid.interfaces import IUUID
from Products.CMFPlone.interfaces.factory import IFactoryTool
from unittest2 import TestCase
from zope.component import getUtility
from zope.interface import alsoProvides
from zope.schema.interfaces import IVocabularyFactory


class TestFtwNotificationIntegration(TestCase):

    layer = FTW_WORKSPACE_INTEGRATION_TESTING

    def setUp(self):

        self.portal = self.layer['portal']
        setRoles(self.portal, TEST_USER_ID, ['Manager'])
        login(self.portal, TEST_USER_NAME)

        self.workspace = create(Builder('workspace').titled('Workspace'))
        self.subfolder = create(Builder('TabbedViewFolder')
                                .within(self.workspace)
                                .titled('Folder1'))
        self.subsubfolder = create(Builder('TabbedViewFolder')
                                   .within(self.subfolder)
                                   .titled('Folder1'))

        self.file_ = create(Builder('file')
                            .titled('A File')
                            .within(self.subsubfolder)
                            .with_dummy_content())

        self.vocab_factory = getUtility(IVocabularyFactory,
                                        name='assignable_users')

        self.owner = [TEST_USER_ID, ]

    def test_list_users_with_local_roles_on_obj(self):
        user1 = create(Builder('user')
                       .named('Test', 'User1')
                       .with_roles('Reader', on=self.workspace))

        self.assertItemsEqual(
            [user1.getId()] + self.owner,
            self.vocab_factory(self.workspace).userids)

    def test_list_users_with_local_roles_on_parents(self):
        user1 = create(Builder('user')
                       .named('Test', 'User1')
                       .with_roles('Reader', on=self.workspace))
        user2 = create(Builder('user')
                       .named('Test', 'User2')
                       .with_roles('Reader', on=self.subfolder))
        user3 = create(Builder('user')
                       .named('Test', 'User3')
                       .with_roles('Reader', on=self.subsubfolder))

        users = map(methodcaller('getId'), [user1, user2, user3])

        self.assertItemsEqual(
            users + self.owner,
            self.vocab_factory(self.file_).userids)

    def test_dont_list_users_with_local_roles_of_childrens(self):
        create(Builder('user')
               .named('Test', 'User1')
               .with_roles('Reader', on=self.workspace))
        user2 = create(Builder('user')
                       .named('Test', 'User2')
                       .with_roles('Reader', on=self.subsubfolder))

        self.assertNotIn(
            user2.getId(),
            self.vocab_factory(self.subfolder).userids)

    def test_do_no_list_users_with_global_roles(self):
        user = create(Builder('user')
                      .named('Test', 'User1')
                      .with_roles('Reader'))

        self.assertNotIn(
            user.getId(),
            self.vocab_factory(self.workspace).userids)

    def test_list_member_of_group_with_local_roles(self):
        user = create(Builder('user')
                      .named('Test', 'User1'))

        create(Builder('group')
               .titled('Group')
               .with_roles('Reader', on=self.workspace)
               .with_members(user))

        self.assertItemsEqual(
            [user.getId()] + self.owner,
            self.vocab_factory(self.subfolder).userids)

    def test_dont_list_users_with_local_roles_if_acquistion_stopped(self):
        create(Builder('user')
               .named('Test', 'User1')
               .with_roles('Reader', on=self.workspace))

        self.subfolder.__ac_local_roles_block__ = True

        self.assertItemsEqual(
            self.owner,
            self.vocab_factory(self.subfolder).userids)

    def test_dont_list_member_of_group_if_acquistion_stopped(self):
        user = create(Builder('user')
                      .named('Test', 'User1'))

        create(Builder('group')
               .titled('Group')
               .with_roles('Reader', on=self.workspace)
               .with_members(user))

        self.subfolder.__ac_local_roles_block__ = True

        self.assertItemsEqual(
            self.owner,
            self.vocab_factory(self.subfolder).userids)

    def test_vocab_is_sorted_by_fullname(self):
        user1 = create(Builder('user')
                       .named('ZZZ', 'User')
                       .with_roles('Reader', on=self.workspace))
        user2 = create(Builder('user')
                       .named('aaa', 'User')
                       .with_roles('Reader', on=self.subfolder))
        user3 = create(Builder('user')
                       .named('ABC', 'User')
                       .with_roles('Reader', on=self.subsubfolder))

        users = map(lambda user: user.getProperty('fullname'),
                    [user2, user3, user1])
        users.insert(0, '')  # The test user has no fullname

        users_from_vocabulary = [user.title for user in self.vocab_factory(
            self.subsubfolder)]

        self.assertItemsEqual(users, users_from_vocabulary)

    def test_vocabulary_can_lists_contacts_too(self):
        user1 = create(Builder('user')
                       .named('ZZZ', 'User')
                       .with_roles('Reader', on=self.workspace))

        contact = create(Builder('contact')
                         .within(self.workspace)
                         .having(firstname='Contact', lastname='User'))

        self.assertItemsEqual(
            [user1.getId()] + self.owner + [IUUID(contact)],
            self.vocab_factory(self.subfolder, membersonly=False).userids)

        self.assertItemsEqual(
            [user1.getId()] + self.owner,
            self.vocab_factory(self.subfolder, membersonly=True).userids)

    def test_vocabulary_ignore_local_roles_of_factory_tool(self):
        user1 = create(Builder('user')
                       .named('Test', 'User1')
                       .with_roles('Reader', on=self.workspace))
        create(Builder('user')
               .named('Test', 'User2')
               .with_roles('Reader', on=self.subfolder))
        create(Builder('user')
               .named('Test', 'User3')
               .with_roles('Reader', on=self.subsubfolder))

        alsoProvides(self.subfolder, IFactoryTool)

        self.assertItemsEqual(
            [user1.getId()] + self.owner,
            self.vocab_factory(self.file_).userids)



from ftw.upgrade import UpgradeStep


class InstallFtwFile(UpgradeStep):
    """Install ftw file.
    """

    def __call__(self):
        version = self.portal_setup.getLastVersionForProfile(
            'ftw.file:default')
        if version is None or version == 'unknown':
            self.setup_install_profile('profile-ftw.file:default')



from datetime import datetime
from datetime import timedelta
from plone.memoize import ram
from Products.Archetypes.utils import OrderedDict
from Products.CMFCore.utils import getToolByName
from zope.component.hooks import getSite
import os.path


def icon(item, value):
    if not item.getIcon:
        return ''
    url_method = lambda: '#'
    site = getSite()
    props = getToolByName(site, 'portal_properties')
    portal_url = getToolByName(site, 'portal_url')
    item_type = item.portal_type
    ftw_worspace = props.get('ftw.workspace_properties', None)
    if not ftw_worspace:
        # fallback
        direct_downloadable_types = ['File', ]
    else:
        direct_downloadable_types = ftw_worspace.getProperty(
            'direct_downloadable_types',
            ['File', ])
    if hasattr(item, 'getURL'):
        url_method = item.getURL
    elif hasattr(item, 'absolute_url'):
        url_method = item.absolute_url
    has_file = item_type in direct_downloadable_types
    # use a icon
    img = u'<img src="%s/%s"/>' % (portal_url(), item.getIcon)

    # link it with either the download (usually "file" field) or with
    # the content default view
    href = url_method()
    if has_file:
        href = os.path.join(href, 'at_download', 'file')
    elif hasattr(item, 'portal_type'):
        # do we need to add /view ?

        types_using_view = props.get('site_properties').getProperty(
            'typesUseViewActionInListings')
        if item_type in types_using_view:
            href = os.path.join(href, 'view')

    return u'<a href="%s">%s</a>' % (href.decode('utf8'), img)

def icon_or_sprite(item, value):
    if item.getIcon:
        return icon(item, value)
    site = getSite()
    plone_utils = getToolByName(site, 'plone_utils')
    return '<div class="spriteIcon contenttype-%s">&nbsp;</div>' % \
        plone_utils.normalizeString(item.portal_type)

@ram.cache(lambda m, i, author: author)
def readable_author(item, author):
    #TODO: terribly inefficient. Make some HelperCommons or something
    site = getSite()
    portal_url = getToolByName(site, 'portal_url')
    if not author:
        return '-'
    name = author
    user = site.acl_users.getUserById(author)
    url = None
    if user is not None:
        name = user.getProperty('fullname', author)
        if not len(name):
            name = author
    if user is None:
        url, name = find_contact_object(item, author)
    if url is None:
        url = "%s/author/%s" % (portal_url(), author)
    return '<a href="%s">%s</a>' % (url, name)


def find_contact_object(item, id_):
    item = item.getObject()
    current = item.aq_inner.aq_explicit
    while current.aq_parent.Type() in ['Workspace']:
        current = current.aq_parent
    brains = item.portal_catalog(id=id_,
                                 Type='Contact',
                                 path='/'.join(current.getPhysicalPath()))
    if len(brains):
        return brains[0].getURL(), brains[0].Title
    return None, id_


def responsible(item, value):
    return ','.join(
        [readable_author(item, r)
            for r in item.getObject().getResponsibility()])


def group_by_date(results):
    grouped_results = OrderedDict()
    grouped_results['today'] = []
    grouped_results['yesterday'] = []
    grouped_results['this_week'] = []
    grouped_results['this_month'] = []
    grouped_results['older'] = []

    today = datetime.today().day
    yesterday = (datetime.today() - timedelta(1)).day
    this_week = int(datetime.today().strftime("%V"))  # isoweek
    this_month = datetime.today().month
    this_year = datetime.today().year

    for result in results:
        modified = result.modified
        if modified.month() == this_month and modified.year() == this_year:
            if modified.day() == today:
                grouped_results['today'].append(result)
            elif modified.day() == yesterday:
                grouped_results['yesterday'].append(result)
            elif int(modified.strftime("%V")) == this_week:
                grouped_results['this_week'].append(result)
            else:
                grouped_results['this_month'].append(result)
        else:
            grouped_results['older'].append(result)
    return grouped_results.items()



from ftw.builder import Builder
from ftw.builder import create
from ftw.workspace.testing import FTW_WORKSPACE_FUNCTIONAL_TESTING
from plone.app.testing import login
from plone.app.testing import logout
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone.testing.z2 import Browser
from Products.CMFCore.utils import getToolByName
from unittest2 import TestCase
from zExceptions import Unauthorized
import transaction


class TestWorkspaceWorkflow(TestCase):

    layer = FTW_WORKSPACE_FUNCTIONAL_TESTING

    def setUp(self):
        self.browser = Browser(self.layer['app'])
        self.portal = self.layer['portal']
        setRoles(self.portal, TEST_USER_ID, ['Manager'])

        self.portal.portal_types['Folder'].allowed_content_types = (
            'Workspace', )

        self.folder = self._setup_projects_folder_with_placeful_workflow()

    def _setup_projects_folder_with_placeful_workflow(self):
        folder = create(Builder('folder'))

        policy_id = 'workspace_policy'

        folder.manage_addProduct['CMFPlacefulWorkflow']\
            .manage_addWorkflowPolicyConfig()
        config = getToolByName(
            folder,
            'portal_placeful_workflow').getWorkflowPolicyConfig(folder)

        config.setPolicyIn(policy=policy_id)
        config.setPolicyBelow(policy=policy_id, update_security=True)
        transaction.commit()

        return folder

    def test_anonymous_user_cannot_access_folder(self):
        logout()

        with self.assertRaises(Unauthorized):
            self.portal.restrictedTraverse(self.folder.getId())

    def test_authenticated_user_can_view_folder(self):
        user = create(Builder('user'))

        logout()
        login(self.portal, user.getId())

        self.assertTrue(
            self.portal.restrictedTraverse(self.folder.getId()),
            'A authenticated user should have access to the folder')

    def test_authenticated_user_CAN_add_workspace(self):
        user = create(Builder('user'))

        logout()
        login(self.portal, user.getId())

        self.assertTrue(
            create(Builder('workspace').within(self.folder)),
            'Authenticated should be able to add a workspace')

    def test_authenticated_user_CANNOT_view_workspace(self):
        workspace = create(Builder('workspace').within(self.folder))
        user = create(Builder('user').with_roles('Authenticated'))

        logout()
        login(self.portal, user.getId())

        with self.assertRaises(Unauthorized):
            self.folder.restrictedTraverse(workspace.getId())

    def test_reader_CAN_view_workspace(self):
        workspace = create(Builder('workspace').within(self.folder))
        user = create(Builder('user').with_roles('Reader', on=workspace))

        logout()
        login(self.portal, user.getId())

        self.assertTrue(
            self.folder.restrictedTraverse(workspace.getId()),
            'A reader should have access to the workspace')

    def test_contributor_CAN_add_content_in_workspace(self):
        workspace = create(Builder('workspace').within(self.folder))
        user = create(Builder('user').with_roles('Contributor', on=workspace))

        logout()
        login(self.portal, user.getId())

        self.assertTrue(
            create(Builder('TabbedViewFolder').within(workspace)),
            'Contributor should be able to add a tabbedviewfolder')

        self.assertTrue(
            create(Builder('file').within(workspace)),
            'Contributor should be able to add a tabbedviewfolder')

    def test_contributor_CAN_delete_his_content_in_workspace(self):
        workspace = create(Builder('workspace').within(self.folder))
        user = create(Builder('user').with_roles('Contributor', on=workspace))

        logout()
        login(self.portal, user.getId())

        item = create(Builder('TabbedViewFolder').within(workspace))
        id_ = item.getId()

        workspace.manage_delObjects([id_])

        self.assertNotIn(id_, workspace.objectIds())

    def test_contributor_CAN_delete_others_content(self):
        workspace = create(Builder('workspace').within(self.folder))
        user = create(Builder('user').with_roles('Contributor', on=workspace))

        item = create(Builder('TabbedViewFolder').within(workspace))
        id_ = item.getId()

        logout()
        login(self.portal, user.getId())

        workspace.manage_delObjects([id_])

        self.assertNotIn(id_, workspace.objectIds())

    def test_contributor_CANNOT_delete_workspace(self):
        workspace = create(Builder('workspace').within(self.folder))
        user = create(Builder('user').with_roles('Contributor', on=workspace))

        logout()
        login(self.portal, user.getId())

        with self.assertRaises(Unauthorized):
            self.folder.manage_delObjects([workspace.getId()])

    def test_admin_CAN_manage_permissions(self):
        workspace = create(Builder('workspace').within(self.folder))
        user = create(Builder('user')
                      .with_roles('Editor', on=workspace))

        logout()
        login(self.portal, user.getId())

        self.assertTrue(workspace.restrictedTraverse('@@sharing'),
                        'Admin shold habe access to the sharing view')

    def test_others_CANNOT_manage_permissions(self):
        workspace = create(Builder('workspace').within(self.folder))
        user = create(Builder('user').with_roles('Contributor', on=workspace))

        logout()
        login(self.portal, user.getId())

        with self.assertRaises(Unauthorized):
            workspace.restrictedTraverse('@@sharing')

    def test_admin_CAN_delete_workspace(self):
        workspace = create(Builder('workspace').within(self.folder))
        workspace_id = workspace.getId()
        user = create(Builder('user')
                      .with_roles('Editor', on=workspace))

        logout()
        login(self.portal, user.getId())

        self.folder.manage_delObjects([workspace.getId()])
        self.assertNotIn(workspace_id, self.folder.objectIds())

    def test_site_admin_CAN_delete_his_workspace(self):
        workspace = create(Builder('workspace').within(self.folder))
        workspace_id = workspace.getId()
        user = create(Builder('user')
                      .with_roles('Editor'))

        logout()
        login(self.portal, user.getId())

        self.folder.manage_delObjects([workspace_id])

        self.assertNotIn(workspace_id, self.folder.objectIds())

    def test_reader_CANNOT_edit_content(self):
        workspace = create(Builder('workspace').within(self.folder))
        user = create(Builder('user').with_roles('Reader', on=workspace))
        item = create(Builder('TabbedViewFolder').within(workspace))

        logout()
        login(self.portal, user.getId())

        with self.assertRaises(Unauthorized):
            item.restrictedTraverse('edit')

    def test_contributor_CAN_edit_his_content(self):
        workspace = create(Builder('workspace').within(self.folder))
        user = create(Builder('user').with_roles('Contributor', on=workspace))

        logout()
        login(self.portal, user.getId())

        item = create(Builder('TabbedViewFolder').within(workspace))
        self.assertTrue(item.restrictedTraverse('edit'),
                        'Contributor should be able to edit his content')

    def test_contributor_CAN_edit_others_content(self):
        workspace = create(Builder('workspace').within(self.folder))
        user = create(Builder('user').with_roles('Contributor', on=workspace))
        item = create(Builder('TabbedViewFolder').within(workspace))

        logout()
        login(self.portal, user.getId())

        self.assertTrue(item.restrictedTraverse('edit'),
                        'Contributor should be able to edit others content')



from ftw.builder import Builder
from ftw.builder import create
from ftw.workspace.interfaces import IWorkspacePreview
from ftw.workspace.testing import FTW_WORKSPACE_INTEGRATION_TESTING
from plone.app.testing import login
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone.app.testing import TEST_USER_NAME
from pyquery import PyQuery
from unittest2 import TestCase
from zope.component import queryMultiAdapter
import os


class TestPreview(TestCase):

    layer = FTW_WORKSPACE_INTEGRATION_TESTING

    def setUp(self):
        portal = self.layer['portal']
        setRoles(portal, TEST_USER_ID, ['Manager'])
        login(portal, TEST_USER_NAME)

        self.workspace = create(Builder('workspace'))
        self.previews = self.workspace.restrictedTraverse(
            '@@previews')

        portal.portal_types.get(
            'Workspace').allowed_content_types = ('File', 'Image')

    def test_get_extension_no_contenttype(self):
        self.assertListEqual([], self.previews.get_extensions(None))

    def test_get_extension_no_mimetype(self):
        self.assertListEqual([],
                             self.previews.get_extensions('dummy/contenttype'))

    def test_get_extension_malformed_mimetype(self):
        self.assertListEqual([],
                             self.previews.get_extensions('dummycontenttype'))

    def test_get_extension_valid_mimetype(self):
        self.assertListEqual(['gif'],
                             list(self.previews.get_extensions('image/gif')))

    def test_get_previews(self):

        create(Builder('image')
            .within(self.workspace)
            .with_dummy_content())

        self.assertGreaterEqual(len(self.previews.get_previews()),
                                1,
                               'Expect at least one adapter')

    def test_default_preview(self):
        file_ = create(Builder('file')
            .within(self.workspace)
            .with_dummy_content())

        adapter = queryMultiAdapter(
            (file_, file_.REQUEST),
            IWorkspacePreview)

        self.assertIn('default.png', adapter.preview())

    def test_gif_preview(self):
        image = create(Builder('image')
            .within(self.workspace)
            .with_dummy_content())

        adapter = queryMultiAdapter(
            (image, image.REQUEST),
            IWorkspacePreview,
            name='gif')

        self.assertTrue(
            adapter.preview().startswith(
                '<img src="http://nohost/plone/workspace/image'),
            'Expect an image tag. source should be our image')

    def test_gif_full_url(self):
        image = create(Builder('image')
            .within(self.workspace)
            .with_dummy_content())

        adapter = queryMultiAdapter(
            (image, image.REQUEST),
            IWorkspacePreview,
            name='gif')

        self.assertEquals(image.absolute_url() + '/images/image',
                         adapter.full_url())

    def test_gif_scale_properties(self):
        image = create(Builder('image')
            .within(self.workspace)
            .with_dummy_content())

        adapter = queryMultiAdapter(
            (image, image.REQUEST),
            IWorkspacePreview,
            name='gif')

        self.assertEquals((200, 200), adapter.get_scale_properties())

    def test_tab_renders(self):
        create(Builder('image')
            .within(self.workspace)
            .with_dummy_content())

        doc = PyQuery(self.workspace.restrictedTraverse(
            '@@tabbedview_view-preview')())

        self.assertTrue(doc('.previewContainer .colorboxLink img'),
                            'There should be an image')

    def test_ftwfile_gif_preview(self):
        image = ('GIF89a\x01\x00\x01\x00\x80\x00\x00\x00\x00\x00\x00\x00'
                '\x00!\xf9\x04\x04\x00\x00\x00\x00,\x00\x00\x00\x00\x01\x00'
                '\x01\x00\x00\x02\x02D\x01\x00;')

        file_ = create(Builder('file')
            .within(self.workspace)
            .attach_file_containing(image))

        adapter = queryMultiAdapter(
            (file_, file_.REQUEST),
            IWorkspacePreview,
            name='gif')

        self.assertTrue(
            adapter.preview().startswith(
                '<img src="http://nohost/plone/workspace/file'),
            'Expect an image tag. source should be our image')

        self.assertEquals((200, 200), adapter.get_scale_properties())

    def test_default_preview_query(self):
        query = self.previews._query()

        path = '/'.join(self.workspace.getPhysicalPath())
        self.assertEquals(path, query['path'])

        self.assertEquals('modified', query['sort_on'])
        self.assertEquals('descending', query['sort_order'])

    def test_default_get_group_information(self):
        image = create(Builder('image')
            .within(self.workspace)
            .with_dummy_content())
        adapter = queryMultiAdapter(
            (image, image.REQUEST),
            IWorkspacePreview,
            name='gif')

        self.assertEquals('heute',
                          self.previews.get_group_information(adapter))

    def test_doc_preview_full_url(self):
        file_content = open("{0}/data/test.doc".format(
            os.path.split(__file__)[0], 'r'))
        file_ = create(Builder('file')
            .within(self.workspace)
            .attach_file_containing(file_content))

        adapter = queryMultiAdapter(
            (file_, file_.REQUEST),
            IWorkspacePreview,
            name='doc')

        self.assertIn('doc.png', adapter.full_url())

    def test_docx_preview_full_url(self):
        file_content = open("{0}/data/test.docx".format(
            os.path.split(__file__)[0], 'r'))
        file_ = create(Builder('file')
            .within(self.workspace)
            .attach_file_containing(file_content))

        adapter = queryMultiAdapter(
            (file_, file_.REQUEST),
            IWorkspacePreview,
            name='docx')

        self.assertIn('docx.png', adapter.full_url())

    def test_ppt_preview_full_url(self):
        file_content = open("{0}/data/test.ppt".format(
            os.path.split(__file__)[0], 'r'))
        file_ = create(Builder('file')
            .within(self.workspace)
            .attach_file_containing(file_content))

        adapter = queryMultiAdapter(
            (file_, file_.REQUEST),
            IWorkspacePreview,
            name='ppt')

        self.assertIn('ppt.png', adapter.full_url())

    def test_pptx_preview_full_url(self):
        file_content = open("{0}/data/test.pptx".format(
            os.path.split(__file__)[0], 'r'))
        file_ = create(Builder('file')
            .within(self.workspace)
            .attach_file_containing(file_content))

        adapter = queryMultiAdapter(
            (file_, file_.REQUEST),
            IWorkspacePreview,
            name='pptx')

        self.assertIn('pptx.png', adapter.full_url())

    def test_xls_preview_full_url(self):
        file_content = open("{0}/data/test.xls".format(
            os.path.split(__file__)[0], 'r'))
        file_ = create(Builder('file')
            .within(self.workspace)
            .attach_file_containing(file_content))

        adapter = queryMultiAdapter(
            (file_, file_.REQUEST),
            IWorkspacePreview,
            name='xls')

        self.assertIn('xls.png', adapter.full_url())

    def test_xlsx_preview_full_url(self):
        file_content = open("{0}/data/test.xlsx".format(
            os.path.split(__file__)[0], 'r'))
        file_ = create(Builder('file')
            .within(self.workspace)
            .attach_file_containing(file_content))

        adapter = queryMultiAdapter(
            (file_, file_.REQUEST),
            IWorkspacePreview,
            name='xlsx')

        self.assertIn('xlsx.png', adapter.full_url())

    def test_pdf_preview_full_url(self):
        file_content = open("{0}/data/test.pdf".format(
            os.path.split(__file__)[0], 'r'))
        file_ = create(Builder('file')
            .within(self.workspace)
            .attach_file_containing(file_content))

        adapter = queryMultiAdapter(
            (file_, file_.REQUEST),
            IWorkspacePreview,
            name='pdf')

        self.assertIn('pdf.png', adapter.full_url())

    def test_zip_preview_full_url(self):
        file_content = open("{0}/data/test.zip".format(
            os.path.split(__file__)[0], 'r'))
        file_ = create(Builder('file')
            .within(self.workspace)
            .attach_file_containing(file_content))

        adapter = queryMultiAdapter(
            (file_, file_.REQUEST),
            IWorkspacePreview,
            name='zip')

        self.assertIn('zip.png', adapter.full_url())

    def test_txt_preview_full_url(self):
        file_content = open("{0}/data/test.txt".format(
            os.path.split(__file__)[0], 'r'))
        file_ = create(Builder('file')
            .within(self.workspace)
            .attach_file_containing(file_content))

        adapter = queryMultiAdapter(
            (file_, file_.REQUEST),
            IWorkspacePreview,
            name='txt')

        self.assertIn('txt.png', adapter.full_url())



from ftw.workspace.browser.overview import ListingHelper
from plone.registry.interfaces import IRegistry
from Products.CMFCore.utils import getToolByName
from Products.Five.browser import BrowserView
from zope.component import getUtility
from zope.i18n import translate


class Sublisting(BrowserView, ListingHelper):
    """Workspace sublisting"""

    def collect(self):
        registry = getUtility(IRegistry)
        types = registry.get('ftw.workspace.sublisting_types',
                             ['TabbedViewFolder'])

        result = []
        portal_props = getToolByName(self.context, 'portal_properties')
        sort_on = portal_props.navtree_properties.sortAttribute

        for type_ in types:
            query = {'portal_type': type_,
                     'sort_on': sort_on}

            objects = self.context.getFolderContents(contentFilter=query)

            if not objects:
                continue

            result.append(
                dict(title=self.translated_title(type_),
                     objects=objects)
            )

            result.sort(key=lambda x: x['title'])

        return result

    def translated_title(self, type_):
        portal_types = getToolByName(self.context, 'portal_types')
        fti = portal_types.get(type_, None)
        if fti is None or not fti.i18n_domain:
            domain = 'plone'
        else:
            domain = fti.i18n_domain

        if fti:
            title = fti.title
        else:
            title = type_

        return translate(msgid=title, domain=domain,
                         context=self.request)



from AccessControl import ClassSecurityInfo
from ftw.workspace.config import PROJECTNAME
from ftw.workspace.content.schemata import finalizeWorkspaceSchema
from ftw.workspace.interfaces import ITabbedViewFolder
from Products.Archetypes import atapi
from Products.ATContentTypes.content import folder
from zope.interface import implements

schema = folder.ATFolder.schema.copy()

finalizeWorkspaceSchema(schema)


class TabbedViewFolder(folder.ATFolder):
    implements(ITabbedViewFolder)

    schema = schema

    security = ClassSecurityInfo()

    security.declarePublic('canSetDefaultPage')
    def canSetDefaultPage(self):
        return False


atapi.registerType(TabbedViewFolder, PROJECTNAME)



from ftw.upgrade import UpgradeStep


class InstallConfigureNotification(UpgradeStep):
    """Install configure notification.
    """

    def __call__(self):
        base_version = self.portal_setup.getLastVersionForProfile(
            'ftw.notification.base:default')
        if base_version is None or base_version == 'unknown':
            self.setup_install_profile('profile-ftw.notification.base:default')

        email_version = self.portal_setup.getLastVersionForProfile(
            'ftw.notification.base:email')
        if email_version is None or email_version == 'unknown':
            self.setup_install_profile('profile-ftw.notification.email:default')

        self.install_upgrade_profile()



from ftw.pdfgenerator.interfaces import IPDFAssembler
from ftw.workspace import arbeitsraumMessageFactory as _
from ftw.workspace.export import get_header, get_data, create_xlsx
from ftw.workspace.interfaces import IWorkspace
from ftw.zipexport.interfaces import IZipRepresentation
from ftw.zipexport.representations.archetypes import FolderZipRepresentation
from Products.CMFPlone.utils import safe_unicode
from StringIO import StringIO
from zope.component import adapts
from zope.component import getMultiAdapter
from zope.i18n import translate
from zope.interface import implements
from zope.interface import Interface


class WorkspaceZipRepresentation(FolderZipRepresentation):
    implements(IZipRepresentation)
    adapts(IWorkspace, Interface)

    def get_files(self, path_prefix=u"", recursive=True, toplevel=True):
        """
        Returns a list of two-value-tuples having the following values:
        - a relative path under which the file should show up in the zip
        - the data as either a file or a stream
        """
        filename = '{0}.pdf'.format(self.context.getId())

        assembler = getMultiAdapter((self.context, self.request),
                                    IPDFAssembler)

        yield (u'{0}/{1}'.format(safe_unicode(path_prefix),
                                 safe_unicode(filename)),
               StringIO(assembler.build_pdf()))

        header = get_header(self.context)
        data = get_data(self.context)
        xlsx = create_xlsx(header, data)

        filename = translate(
            _(u'participants_export_filename', default=u'participants.xlsx'),
            context=self.request
        )

        yield (u'{0}/{1}'.format(safe_unicode(path_prefix),
                                 safe_unicode(filename)), xlsx)
        
        # Recursively export folder contents.
        folder_contents = super(WorkspaceZipRepresentation, self).get_files(
            path_prefix, recursive, toplevel)

        for item in folder_contents:
            yield item



from ftw.workspace import _
from ftw.workspace.interfaces import IWorkspacePreview
from plone.namedfile.interfaces import IAvailableSizes
from Products.CMFCore.utils import getToolByName
from zope.component import adapts
from zope.component import getUtility
from zope.interface import implements
from zope.interface import Interface


class DefaultPreview(object):
    """Default preview"""

    implements(IWorkspacePreview)
    adapts(Interface, Interface)

    def __init__(self, context, request):
        self.context = context
        self.request = request

    def preview(self):
        return '<img height="200px" src="{0}" alt={1} title={1}/>'.format(
            self.full_url(),
            _(u'text_no_preview', default=u'No Preview')
            )

    def full_url(self):
        portal_url = getToolByName(self.context, 'portal_url')
        return '{0}/++resource++ftw.workspace-resources/default.png'.format(
            portal_url())

    def get_scale_properties(self):
        sizes = getUtility(IAvailableSizes)()
        return sizes.get('workspace_preview', (200, 200))



from ftw.builder import Builder
from ftw.builder import create
from ftw.testbrowser import browser
from ftw.testbrowser import browsing
from ftw.workspace.testing import FTW_WORKSPACE_FUNCTIONAL_TESTING
from plone.app.testing import login
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone.app.testing import TEST_USER_NAME
from unittest2 import TestCase


class TestFtwNotificationIntegration(TestCase):

    layer = FTW_WORKSPACE_FUNCTIONAL_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        setRoles(self.portal, TEST_USER_ID, ['Manager'])
        login(self.portal, TEST_USER_NAME)

        self.workspace = create(Builder('workspace').titled('Workspace'))

    @browsing
    def test_notification_action_on_workspace(self, browser):
        self.assert_notification_action(self.workspace)

    @browsing
    def test_notification_action_on_tabbedviewfolder(self, browser):
        folder = create(Builder('TabbedViewFolder').within(self.workspace))
        self.assert_notification_action(folder)

    @browsing
    def test_notification_action_on_file(self, browser):
        file_ = create(Builder('file').within(self.workspace))
        self.assert_notification_action(file_)

    def assert_notification_action(self, obj):
        browser.login().visit(obj)
        self.assertEquals(
            'Notification',
            browser.css('#plone-contentmenu-actions-notification').first.text)



from ftw.tabbedview.browser import listing


class Tab(listing.CatalogListingView):
    """Base view class for workspace tabs.

    See ftw.tabbedview.browser.views.BaseListingView for attributes and
    methods that are meant to be overwritten to change the behavior.
    """



"""Definition of the Workspace content type
"""

from AccessControl import ClassSecurityInfo
from Products.ATContentTypes.content import folder
from Products.Archetypes import atapi
from Products.Archetypes.interfaces import IObjectInitializedEvent
from Products.CMFCore.utils import getToolByName
from ftw.workspace import _
from ftw.workspace.config import PROJECTNAME
from ftw.workspace.content.schemata import finalizeWorkspaceSchema
from ftw.workspace.interfaces import IWorkspace
from ftw.workspace.utils import get_creator_fullname
from ftw.workspace.utils import TinyMCEAllowedButtonsConfigurator
from plone.registry.interfaces import IRegistry
from zope.component import adapter
from zope.component import getUtility
from zope.interface import implements


WorkspaceSchema = folder.ATFolderSchema.copy() + atapi.Schema((
    atapi.TextField(
        'text',
        searchable=True,
        required=False,
        allowable_content_types=('text/html', ),
        default_content_type='text/html',
        validators=('isTidyHtmlWithCleanup', ),
        default_output_type='text/x-html-safe',
        default_input_type='text/html',
        storage=atapi.AnnotationStorage(),
        widget=atapi.RichWidget(
            label=_(u"label_text", default=u"Text"),
            rows=15,
            allow_file_upload=False,
            allow_buttons=TinyMCEAllowedButtonsConfigurator(),
        ),
    ),
))


finalizeWorkspaceSchema(WorkspaceSchema,
                        folderish=True,
                        moveDiscussion=False)


@adapter(IWorkspace, IObjectInitializedEvent)
def workspace_added(object_, event):
    """When a workspace is created, we add additional local roles
    to enable the creator to delegate them.
    """
    pm_tool = getToolByName(object_, 'portal_membership')
    current_user = pm_tool.getAuthenticatedMember().getId()
    # Fix (PHa): Only set local roles for logged-in user,
    # if his/her id already has local roles
    if current_user in object_.__ac_local_roles__:
        registry = getUtility(IRegistry)
        roles = registry['ftw.workspace.auto_roles']
        object_.__ac_local_roles__[current_user] += roles


class Workspace(folder.ATFolder):
    """A type for collaborative spaces."""
    implements(IWorkspace)

    portal_type = "Workspace"
    schema = WorkspaceSchema

    security = ClassSecurityInfo()

    security.declarePublic('canSetDefaultPage')
    def canSetDefaultPage(self):
        return False

    security.declarePublic('SearchableText')
    def SearchableText(self):
        fullname = get_creator_fullname(self)
        return ' '.join((super(Workspace, self).SearchableText(), fullname))

atapi.registerType(Workspace, PROJECTNAME)



from archetypes.schemaextender.field import ExtensionField
from archetypes.schemaextender.interfaces import IOrderableSchemaExtender
from ftw.workspace import _
from ftw.workspace.utils import TinyMCEAllowedButtonsConfigurator
from plone.registry.interfaces import IRegistry
from Products.Archetypes.public import TextField, RichWidget
from Products.ATContentTypes.content.folder import ATFolder
from zope.component import adapts
from zope.component import queryUtility
from zope.interface import implements


class folderTextField(ExtensionField, TextField):
    """ Extension TextField
    """


class FolderExtender(object):
    adapts(ATFolder)
    implements(IOrderableSchemaExtender)

    fields = [folderTextField(
        'text',
        searchable=True,
        required=False,
        allowable_content_types=('text/html', ),
        default_content_type='text/html',
        validators=('isTidyHtmlWithCleanup', ),
        default_output_type='text/x-html-safe',
        default_input_type='text/html',
        widget=RichWidget(
            label=_(u"label_text", default=u"Text"),
            rows=15,
            allow_file_upload=False,
            allow_buttons=TinyMCEAllowedButtonsConfigurator(), )), ]

    def __init__(self, context):
        self.context = context

    def getFields(self):

        # self.context must AQ wrapped
        if not getattr(self.context, 'aq_parent', None):
            return []

        registry = queryUtility(IRegistry)
        if not registry:
            return []
        show = False
        if 'ftw.workspace.showtextfieldonfolder' in registry:
            show = registry['ftw.workspace.showtextfieldonfolder']

        if show and self.context.portal_type == 'TabbedViewFolder':
            return self.fields
        return []

    def getOrder(self, original):
        return original



from ftw.table import helper
from ftw.workspace import _
from ftw.workspace.browser import helper as workspace_helper
from ftw.workspace.browser.tab import Tab
from ftw.workspace.utils import has_ftwfile
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from zope.i18nmessageid import MessageFactory


fileMF = MessageFactory('ftw.file')


class DocumentsTab(Tab):

    types = ['File', 'Document', 'Image']

    show_selects = False
    show_menu = True

    sort_reverse = True

    template = ViewPageTemplateFile('documents.pt')

    enabled_actions = ['delete', 'move_items']

    def __init__(self, context, request):
        super(DocumentsTab, self).__init__(context, request)
        # default implementation
        if has_ftwfile(self.context):
            # ftw.file implementation
            self.sort_on = 'documentDate'
        else:
            self.sort_on = 'effective'

    @property
    def columns(self):
        # default implementation
        date_column = {'column': 'effective',
                       'column_title': _(u'column_date', default=u'date'),
                       'transform': helper.readable_date_text,
                       'width': 100}
        # ftw.file implementation
        if has_ftwfile(self.context):
            date_column['column'] = 'documentDate'
            date_column['column_title'] = fileMF(u'label_document_date',
                                                 default=u'Document date')

        columns = (
            {'column': '',
             'column_title': '',
             'transform': helper.path_checkbox,
             'sortable': False,
             'width': 30},

            {'column': 'getIcon',
             'sort_index': 'getContentType',
             'column_title': _(u'column_type', default=u'Type'),
             'transform': workspace_helper.icon_or_sprite,
             'width': 35},

            {'column': 'Title',
             'column_title': _(u'column_title', default=u'Title'),
             'sort_index': 'sortable_title',
             'transform': helper.link(icon=False, tooltip=True)},
        ) + (date_column,) + (  # Add variable date column
            {'column': 'Creator',
             'column_title': _(u'column_creator', default=u'Creator'),
             'sort_index': 'sortable_creator',
             'transform': helper.readable_author},

            {'column': 'modified',
             'column_title': _(u'column_modified', default=u'modified'),
             'transform': helper.readable_date_text,
             'width': 80},
        )
        return columns



from ftw.builder import Builder
from ftw.builder import create
from ftw.workspace.testing import FTW_WORKSPACE_INTEGRATION_TESTING
from plone.app.testing import login
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone.app.testing import TEST_USER_NAME
from plone.registry.interfaces import IRegistry
from pyquery import PyQuery
from unittest2 import TestCase
from zope.component import getUtility
from Products.CMFCore.utils import getToolByName

class TestOverviewFolderSublisting(TestCase):

    layer = FTW_WORKSPACE_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        setRoles(self.portal, TEST_USER_ID, ['Manager'])
        login(self.portal, TEST_USER_NAME)

        self.workspace = create(Builder('workspace'))
        self.view = self.workspace.restrictedTraverse('@@overview_sublisting')

    def test_sublisting_empty(self):

        self.assertListEqual([], self.view.collect())

    def test_sublisting_folders(self):
        create(Builder('TabbedViewFolder')
            .within(self.workspace)
            .titled('Folder'))

        self.assertEquals(1, len(self.view.collect()), 'Expect one Folder')

    def test_sublisting_result(self):
        folder1 = create(Builder('TabbedViewFolder')
            .within(self.workspace)
            .titled('A folder'))
        folder2 = create(Builder('TabbedViewFolder')
            .within(self.workspace)
            .titled('B folder'))

        self.assertEquals(1, len(self.view.collect()), 'Expect one Folder')
        self.assertEquals('Folder', self.view.collect()[0]['title'])

        objects = self.view.collect()[0]['objects']
        self.assertIn(folder1.getId(), [item.getId for item in objects])
        self.assertIn(folder2.getId(), [item.getId for item in objects])

    def test_list_more_types(self):
        registry = getUtility(IRegistry)
        registry['ftw.workspace.sublisting_types'].append('File')

        folder = create(Builder('TabbedViewFolder')
            .within(self.workspace)
            .titled('A folder'))

        file_ = create(Builder('file')
            .titled('File')
            .within(self.workspace))

        self.assertEquals(2, len(self.view.collect()), 'Expect two Folders')

        self.assertEquals('File', self.view.collect()[0]['title'])
        self.assertEquals('Folder', self.view.collect()[1]['title'])

        files = self.view.collect()[0]['objects']
        folders = self.view.collect()[1]['objects']

        self.assertIn(folder.getId(), [item.getId for item in folders])
        self.assertIn(file_.getId(), [item.getId for item in files])

    def test_sublisting_renders(self):
        create(Builder('TabbedViewFolder')
            .within(self.workspace)
            .titled('A folder'))

        doc = PyQuery(self.view())

        self.assertTrue(doc('ul li a'))
        self.assertTrue(doc('h2'))

    def test_sublisting_renders_in_overview_tab(self):
        create(Builder('TabbedViewFolder')
            .within(self.workspace)
            .titled('Folder'))

        tab = self.workspace.restrictedTraverse('@@tabbedview_view-overview')
        doc = PyQuery(tab())

        self.assertTrue(doc('.box.sublisting li'))

    def test_sublisting_listed_objects_are_linked(self):
        folder = create(Builder('TabbedViewFolder')
            .within(self.workspace)
            .titled('A folder'))

        doc = PyQuery(self.view())
        href = doc('.box.sublisting a.rollover').attr('href')

        self.assertEquals(folder.absolute_url(), href)

    def test_translate_title(self):
        self.assertEquals('Folder',
                          self.view.translated_title('TabbedViewFolder'))
        self.assertEquals('Page',
                          self.view.translated_title('Document'))
        self.assertEquals('File',
                          self.view.translated_title('File'))

    def test_translate_title_fallback(self):
        self.assertEquals('Dummy',
                          self.view.translated_title('Dummy'))

    def test_sublisting_sorting_correct_obj_position_in_parent(self):
        create(Builder('TabbedViewFolder')
            .within(self.workspace)
            .titled('B folder'))
        create(Builder('TabbedViewFolder')
            .within(self.workspace)
            .titled('A folder'))
        
        objects = self.view.collect()[0]['objects']
        self.assertEquals('B folder', objects[0].Title)
        self.assertEquals('A folder', objects[1].Title)

    def test_sublisting_sortin_correct_sortable_title(self):
        portal_props = getToolByName(self.portal, 'portal_properties')
        portal_props.navtree_properties.sortAttribute = 'sortable_title'
        create(Builder('TabbedViewFolder')
            .within(self.workspace)
            .titled('B folder'))
        create(Builder('TabbedViewFolder')
            .within(self.workspace)
            .titled('A folder'))

        objects = self.view.collect()[0]['objects']
        self.assertEquals('A folder', objects[0].Title)
        self.assertEquals('B folder', objects[1].Title)



from ftw.upgrade import ProgressLogger
from ftw.upgrade import UpgradeStep
from ftw.workspace.content.folder import TabbedViewFolder
import logging

LOG = logging.getLogger('ftw.workspace.upgrades')


class UseClasseForFolder(UpgradeStep):

    def __call__(self):
        self.update_fti()
        self.migrate_existing_folders()

    def update_fti(self):
        LOG.info('TabbedViewFolder FTI: update factory')
        self.setup_install_profile(
            'profile-ftw.workspace.upgrades:1501')

    def migrate_existing_folders(self):
        catalog = self.getToolByName('portal_catalog')
        brains = catalog.unrestrictedSearchResults(
            portal_type='TabbedViewFolder')

        with ProgressLogger('Migrate TabbedViewFolder classes',
                            brains) as step:
            for brain in brains:
                obj = self.portal.unrestrictedTraverse(brain.getPath())
                self.migrate_class(obj, TabbedViewFolder)
                step()



from AccessControl import Unauthorized
from Products.CMFPlone.utils import base_hasattr
from Acquisition import aq_inner
from ZODB.POSException import ConflictError
from collective.quickupload import logger
from collective.quickupload.browser.uploadcapable import upload_lock
from collective.quickupload.interfaces import IQuickUploadFileSetter
from plone.i18n.normalizer.interfaces import IIDNormalizer
from plone.registry.interfaces import IRegistry
from zope import component
from zope.container.interfaces import INameChooser
from zope.component import getUtility
import transaction


def generate_id(name, context):
    normalizer = component.getUtility(IIDNormalizer)
    chooser = INameChooser(context)

    normalized = normalizer.normalize(name)
    normalized = normalized.replace('_', '-').replace(' ', '-').lower()

    return chooser.chooseName(normalized, context)


class WorkspaceQuickUploadCapableFileFactory(object):
    """Workspace specific Quick upload Adapter"""

    def __init__(self, context):
        self.context = aq_inner(context)

    def __call__(
        self, filename, title, description, content_type, data, portal_type):
        context = aq_inner(self.context)
        error = ''
        result = {}
        result['success'] = None
        charset = context.getCharset()
        name = filename.decode(charset)
        name, ext = name.rsplit('.', 1)

        # check the given portal type with the
        # qu addable types from the registry
        registry = getUtility(IRegistry)
        upload_addable = registry.get(
            'ftw.tabbedview.interfaces.ITabbedView.quickupload_addable_types')

        if portal_type not in upload_addable and upload_addable:
            portal_type = upload_addable[0]

        newid = generate_id(name, self.context)
        title = name

        # copied from collective.quickupload
        upload_lock.acquire()
        try:
            transaction.begin()
            try:
                context.invokeFactory(
                    type_name=portal_type,
                    id=newid,
                    title=title,
                    description=description)

            except Unauthorized:
                error = u'serverErrorNoPermission'
            except ConflictError:
                # rare with xhr upload / happens sometimes with flashupload
                error = u'serverErrorZODBConflict'
            except Exception, e:
                error = u'serverError'
                logger.exception(e)

            if error:
                error = u'serverError'
                logger.info("An error happens with setId from filename, "
                            "the file has been created with a bad id, "
                            "can't find %s", newid)
            else:
                obj = getattr(context, newid)
                if obj:
                    error = IQuickUploadFileSetter(obj).set(
                        data, filename, content_type)

                if base_hasattr(obj, 'processForm'):
                    # AT: includes reindexing the object.
                    obj.processForm()
                else:
                    # Dexterity
                    obj.reindexObject()

            #@TODO : rollback if there has been an error
            transaction.commit()
        finally:
            upload_lock.release()

        result['error'] = error
        if not error:
            result['success'] = obj
        return result



from DateTime import DateTime
from ftw.table.helper import readable_date_text
from ftw.workspace.interfaces import IWorkspacePreview
from plone.batching.batch import BaseBatch
from Products.CMFCore.utils import getToolByName
from Products.Five.browser import BrowserView
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from Products.MimetypesRegistry.common import MimeTypeException
from zope.component import queryMultiAdapter


class PreviewTab(BrowserView):
    """Preview tab for workspace"""

    template = ViewPageTemplateFile('preview_tab.pt')

    def __call__(self):
        return self.template()

    def previews(self):
        return self.context.restrictedTraverse('@@previews')()


class LoadPreviews(BrowserView):

    template = ViewPageTemplateFile('previews.pt')

    def __init__(self, context, request):
        self.context = context
        self.request = request
        self.bsize = 12
        self.bstart = 0

    def __call__(self, bstart=0):
        self.bstart = int(bstart)

        return self.template()

    def _query(self, **kwargs):
        query = dict(
            sort_on='modified',
            sort_order="descending",
            portal_type=['File', 'Document', 'Image'],
            path='/'.join(self.context.getPhysicalPath()))

        query.update(kwargs)
        return query

    def get_extensions(self, contenttype):
        mimetool = getToolByName(self.context, 'mimetypes_registry')

        if not contenttype:
            return []

        try:
            mimetype = mimetool.lookup(contenttype)
        except MimeTypeException:
            return []

        if not mimetype:
            return []
        else:
            # only use first result
            mimetype = mimetype[0]

        return mimetype.extensions

    def get_previews(self, **kwargs):
        catalog = getToolByName(self.context, 'portal_catalog')
        result = catalog(self._query(**kwargs))

        batch = BaseBatch(result, self.bsize, start=self.bstart)
        previews = []

        if self.bstart >= batch.end:
            return ''

        for brain in batch:
            obj = brain.getObject()

            adapter = queryMultiAdapter((obj, obj.REQUEST), IWorkspacePreview)

            # Try to get a specific preview adapter
            for extension in self.get_extensions(obj.getContentType()):
                specific = queryMultiAdapter(
                    (obj, obj.REQUEST),
                    IWorkspacePreview,
                    name=extension)

                if specific is not None:
                    adapter = specific
                    break

            previews.append(adapter)
        return previews

    def get_group_information(self, preview_adapter):
        """Rendered as hidden field per entry"""
        attr = self._query()['sort_on']
        context = preview_adapter.context

        value = getattr(context, attr)()
        if isinstance(value, DateTime):
            return readable_date_text(context, value)
        else:
            return value
